package sun.security.provider;

import java.security.DigestException;
import java.security.ProviderException;
import java.util.Arrays;
import java.security.MessageDigestSpi;

abstract class DigestBase extends MessageDigestSpi implements Cloneable
{
    private byte[] oneByte;
    private final String algorithm;
    private final int digestLength;
    private final int blockSize;
    byte[] buffer;
    private int bufOfs;
    long bytesProcessed;
    static final byte[] padding;
    
    DigestBase(String algorithm, int digestLength, int blockSize) {
        this.algorithm = algorithm;
        this.digestLength = digestLength;
        this.blockSize = blockSize;
        this.buffer = new byte[blockSize];
    }
    
    @Override
    protected final int engineGetDigestLength() {
        return this.digestLength;
    }
    
    @Override
    protected final void engineUpdate(byte b) {
        if (this.oneByte == null) {
            this.oneByte = new byte[1];
        }
        this.oneByte[0] = b;
        this.engineUpdate(this.oneByte, 0, 1);
    }
    
    @Override
    protected final void engineUpdate(byte[] array, int implCompressMultiBlock, int n) {
        if (n == 0) {
            return;
        }
        if (implCompressMultiBlock < 0 || n < 0 || implCompressMultiBlock > array.length - n) {
            throw new ArrayIndexOutOfBoundsException();
        }
        if (this.bytesProcessed < 0L) {
            this.engineReset();
        }
        this.bytesProcessed += (long)n;
        if (this.bufOfs != 0) {
            int min = Math.min(n, this.blockSize - this.bufOfs);
            System.arraycopy(array, implCompressMultiBlock, this.buffer, this.bufOfs, min);
            this.bufOfs += min;
            implCompressMultiBlock += min;
            n -= min;
            if (this.bufOfs >= this.blockSize) {
                this.implCompress(this.buffer, 0);
                this.bufOfs = 0;
            }
        }
        if (n >= this.blockSize) {
            int n2 = implCompressMultiBlock + n;
            implCompressMultiBlock = this.implCompressMultiBlock(array, implCompressMultiBlock, n2 - this.blockSize);
            n = n2 - implCompressMultiBlock;
        }
        if (n > 0) {
            System.arraycopy(array, implCompressMultiBlock, this.buffer, 0, n);
            this.bufOfs = n;
        }
        return;
    }
    
    private int implCompressMultiBlock(byte[] array, int i, int n) {
        while (i <= n) {
            this.implCompress(array, i);
            i += this.blockSize;
        }
        return i;
    }
    
    @Override
    protected final void engineReset() {
        if (this.bytesProcessed == 0L) {
            return;
        }
        this.implReset();
        this.bufOfs = 0;
        this.bytesProcessed = 0L;
        Arrays.fill(this.buffer, (byte)0);
        return;
    }
    
    @Override
    protected final byte[] engineDigest() {
        byte[] array = new byte[this.digestLength];
        try {
            this.engineDigest(array, 0, array.length);
        }
        catch (DigestException cause) {
            throw (ProviderException)new ProviderException("Internal error").initCause(cause);
        }
        return array;
    }
    
    @Override
    protected final int engineDigest(byte[] array, int n, int n2) throws DigestException {
        if (n2 < this.digestLength) {
            throw new DigestException("Length must be at least " + this.digestLength + " for " + this.algorithm + "digests");
        }
        if (n < 0 || n2 < 0 || n > array.length - n2) {
            throw new DigestException("Buffer too short to store digest");
        }
        if (this.bytesProcessed < 0L) {
            this.engineReset();
        }
        this.implDigest(array, n);
        this.bytesProcessed = -1L;
        return this.digestLength;
    }
    
    abstract void implCompress(byte[] p0, int p1);
    
    abstract void implDigest(byte[] p0, int p1);
    
    abstract void implReset();
    
    @Override
    public Object clone() throws CloneNotSupportedException {
        DigestBase digestBase = (DigestBase)super.clone();
        digestBase.buffer = (byte[])digestBase.buffer.clone();
        return digestBase;
    }
    
    static {
        (padding = new byte[136])[0] = -128;
    }
}
